@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright Duncan Macleod 2017
+# Copyright (C) Cardiff University, 2018-2020
 #
 # This file is part of GWOSC.
 #
@@ -16,10 +16,27 @@
 # You should have received a copy of the GNU General Public License
 # along with GWOSC.  If not, see <http://www.gnu.org/licenses/>.
 
+"""
+`gwosc.timeline` provides functions to find segments for a given dataset.
+
+You can search for Timeline segments, based on a flag name, and a
+GPS time interval as follows:
+
+>>> from gwosc.timeline import get_segments
+>>> get_segments('H1_DATA', 1126051217, 1126151217)
+[(1126073529, 1126114861), (1126121462, 1126123267), (1126123553, 1126126832), (1126139205, 1126139266), (1126149058, 1126151217)]
+
+The output is a `list` of ``[start, end)`` 2-tuples which each
+represent a semi-open time interval.
+
+For documentation on what flags are available, for example for the O1
+science run, see `the O1 data release
+page <https://gw-openscience.org/O1/>`__ (*Data Quality*).
+"""  # noqa: E501
+
 from operator import itemgetter
 
-from . import api
-from .urls import sieve
+from . import (api, datasets)
 
 
 def get_segments(flag, start, end, host=api.DEFAULT_URL):
@@ -58,53 +75,55 @@ def timeline_url(flag, start, end, host=api.DEFAULT_URL):
 
 
 def _find_dataset(start, end, detector, host=api.DEFAULT_URL):
-    meta = api.fetch_dataset_json(start, end, host=host)
-    duration = end-start
 
+    # -- utility methods to resolve a (proper) dataset name and its segment
+
+    def _event_segment(event):
+        raw = api._fetch_allevents_event_json(event, host=host)["events"]
+        dataset, meta = list(raw.items())[0]
+        return (
+            dataset,
+            datasets.event_segment(
+                dataset,
+                catalog=meta["catalog.shortName"],
+                version=meta["version"],
+                host=host,
+            ),
+        )
+
+    def _run_segment(run):
+        return run, datasets.run_segment(run, host=host)
+
+    # -- match datasets
+
+    dataset_segments = [
+        ("event", _event_segment),
+        ("run", _run_segment),
+    ]
+
+    duration = end-start
     epochs = []
 
-    for type_ in meta:
-        for epoch, metadata in meta[type_].items():
-            if epoch == 'tenyear':  # tenyear doesn't work with json API
-                continue
-
-            # filter on detector
-            if detector not in metadata['detectors']:
-                continue
-
-            # get dataset segment
-            if type_ == 'events':
-                segment = _event_segment(epoch, host=host, detector=detector)
-            elif type_ == 'runs':
-                segment = metadata['GPSstart'], metadata['GPSend']
-            else:
-                raise ValueError(
-                    "Unrecognised dataset type {!r}".format(type_))
-
-            # compare with request
+    for dstype, _dataset_segment in dataset_segments:
+        for dataset in datasets._iter_datasets(
+                type=dstype,
+                detector=detector,
+                segment=(start, end),
+                host=host,
+        ):
+            try:
+                dataset, segment = _dataset_segment(dataset)
+            except ValueError as exc:
+                if str(exc) == (
+                    "no event datasets found for {!r}".format(dataset)
+                ):
+                    continue
+                raise
             overlap = min(end, segment[1]) - max(start, segment[0])
-            epochs.append((epoch, duration-overlap))
+            epochs.append((dataset, duration-overlap))
 
     if not epochs:
         raise ValueError(
             "No datasets found matching [{}, {})".format(start, end))
 
     return sorted(epochs, key=itemgetter(1, 0))[0][0]
-
-
-def _event_segment(event, host=api.DEFAULT_URL, **match):
-    jdata = api.fetch_event_json(event, host=host)
-    seg = None
-    for fmeta in sieve(jdata['strain'], **match):
-        start = fmeta['GPSstart']
-        end = start + fmeta['duration']
-        if seg is None:  # first segment
-            seg = (start, end)
-            continue
-        seg = min(start, seg[0]), max(end, seg[1])
-
-    # fail on no match
-    if seg is None:
-        raise ValueError("No files matched for event {}".format(event))
-
-    return seg
